@@ -49,54 +49,73 @@ setClassUnion("MriImageData", c("SparseArray","array","NULL"))
 #' @field source String naming the file(s) that the image was read from. This
 #'   is reset to the empty string if the image is modified
 #' @field origin Numeric vector giving the spatial coordinate origin
-#' @field storedXform Numeric matrix giving the NIfTI xform matrix read from
-#'   file, if any
+#' @field xform Numeric matrix giving the NIfTI-style xform matrix associated
+#'   with the image, which indicates its orientation
 #' @field reordered Logical value indicating whether the image has been
 #'   reordered. See \code{\link{reorderMriImage}}
 #' @field tags Named list of arbitrary DICOM-style tags
 #' @field data Sparse or dense array of data, or \code{NULL}
 #' 
 #' @export
-MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(imageDims="integer",voxelDims="numeric",voxelDimUnits="character",source="character",origin="numeric",storedXform="matrix",reordered="logical",tags="list",data="MriImageData"), methods=list(
-    initialize = function (imageDims = NULL, voxelDims = NULL, voxelDimUnits = NULL, source = "", origin = NULL, storedXform = emptyMatrix(), reordered = TRUE, tags = list(), data = NULL, ...)
+MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(imageDims="integer",voxelDims="numeric",voxelDimUnits="character",source="character",origin="numeric",xform="matrix",reordered="logical",tags="list",data="MriImageData"), methods=list(
+    initialize = function (imageDims = NULL, voxelDims = NULL, voxelDimUnits = NULL, source = "", origin = NULL, xform = emptyMatrix(), reordered = TRUE, tags = list(), data = NULL, ...)
     {
-        if (is.null(voxelDimUnits))
-            voxelDimUnits <- "unknown"
+        oldFields <- list(...)
         
-        # For backwards compatibility
-        if (source == "internal")
-            source <- ""
-        
-        # For backwards compatibility
-        if (length(tags) == 2 && all(c("keys","values") %in% names(tags)))
-            tags <- structure(as.list(tags$values), names=tags$keys)
-        
-        if (!is.null(imageDims) && !is.null(data) && !equivalent(imageDims,dim(data)))
-            dim(data) <- imageDims
-        else if (is.null(imageDims))
-            imageDims <- dim(data)
-        
-        if (length(origin) < 3)
-            origin <- c(as.numeric(origin), rep(0,3-length(origin)))
-        else
-            origin <- as.numeric(origin)[1:3]
-        
-        # For backwards compatibility
-        if (is.null(voxelDims) && "voxdims" %in% names(list(...)))
+        # Resolve image dimensions
+        .dims <- imageDims
+        if (is.null(imageDims))
         {
-            oldFields <- list(...)
-            if (is.null(oldFields$voxunit))
-                oldFields$voxunit <- "unknown"
-            object <- initFields(imageDims=as.integer(oldFields$imagedims), voxelDims=abs(as.numeric(oldFields$voxdims)), voxelDimUnits=oldFields$voxunit, source=source, origin=origin, storedXform=storedXform, reordered=reordered, tags=tags, data=data)
+            if ("imagedims" %in% names(oldFields))
+                .dims <- oldFields$imagedims
+            else
+                .dims <- dim(data)
         }
-        else
-            object <- initFields(imageDims=as.integer(imageDims), voxelDims=abs(as.numeric(voxelDims)), voxelDimUnits=voxelDimUnits, source=source, origin=origin, storedXform=as.matrix(storedXform), reordered=reordered, tags=tags, data=data)
         
-        if (length(object$imageDims) != length(object$voxelDims))
+        # Sanity checking
+        if (!is.null(.dims) && length(.dims) < 2)
+            report(OL$Error, "Image must be multidimensional")
+        if (!is.null(data) && !is.null(dim(data)) && !equivalent(.dims,dim(data)))
+            flag(OL$Warning, "Data dimensions do not match the specified image dimensions")
+        
+        # Resolve voxel dimensions and associated units
+        .voxdims <- voxelDims
+        .voxunits <- as.character(voxelDimUnits)
+        if (is.null(voxelDims) && "voxdims" %in% names(oldFields))
+            .voxdims <- oldFields$voxdims
+        if (is.null(voxelDimUnits) && "voxunit" %in% names(oldFields))
+            .voxunits <- as.character(oldFields$voxunit)
+        
+        # Name voxel units
+        names(.voxunits)[.voxunits %~% "m$"] <- "spatial"
+        names(.voxunits)[.voxunits %~% "s$"] <- "temporal"
+        
+        # Resolve xform
+        .xform <- xform
+        if (is.emptyMatrix(xform) && "storedXform" %in% names(oldFields))
+            .xform <- oldFields$storedXform
+        
+        if (length(.dims) != length(.voxdims))
             report(OL$Error, "Image and voxel dimensions should have the same length")
         
-        names(object$voxelDimUnits)[object$voxelDimUnits %~% "m$"] <- "spatial"
-        names(object$voxelDimUnits)[object$voxelDimUnits %~% "s$"] <- "temporal"
+        # Don't call as.character() on .voxunits here, as it will drop names
+        object <- initFields(imageDims=as.integer(.dims), voxelDims=abs(as.numeric(.voxdims)), voxelDimUnits=.voxunits, source=as.character(source), origin=as.numeric(origin), xform=as.matrix(.xform), reordered=reordered, tags=tags, data=data)
+        
+        # Make sure the data dimensions are right
+        if (!is.null(object$data))
+            dim(object$data) <- .dims
+        
+        # Various fix-ups for backwards compatibility
+        if (length(object$voxelDimUnits) == 0)
+            object$voxelDimUnits <- "unknown"
+        if (length(object$source) != 1 || object$source == "internal")
+            object$source <- ""
+        if (length(object$tags) == 2 && all(c("keys","values") %in% names(object$tags)))
+            object$tags <- structure(as.list(object$tags$values), names=object$tags$keys)
+        if (length(object$origin) < 3)
+            object$origin <- c(object$origin, rep(0,3-length(object$origin)))
+        else
+            object$origin <- object$origin[1:3]
         
         return (object)
     },
@@ -122,10 +141,46 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
     {
         "Fill the image with a particular value"
         if (value == 0)
-            .self$data <- newSparseArrayWithData(vector(class(value),0), matrix(NA,0,length(imageDims)), imageDims)
+            .self$setData(newSparseArrayWithData(vector(class(value),0), matrix(NA,0,length(imageDims)), imageDims))
         else
-            .self$data <- array(value, dim=imageDims)
-        .self$setSource(NULL)
+            .self$setData(array(value, dim=imageDims))
+    },
+    
+    find = function (fun = NULL, ..., array = TRUE)
+    {
+        "Find voxels whose values are not zero, or satisfy a function"
+        .warnIfIndexingUnreorderedImage(.self)
+        
+        if (.self$isEmpty())
+            return (integer(0))
+        else if (is.null(fun))
+        {
+            if (.self$isSparse())
+            {
+                locs <- data$getCoordinates()
+                if (array)
+                    return (locs)
+                else
+                    return (matrixToVectorLocs(locs, imageDims))
+            }
+            else
+                return (which(data != 0, arr.ind=array))
+        }
+        else
+        {
+            if (is.numeric(fun) && length(fun) == 1)
+                locs <- which(as.array(data) == fun)
+            else
+            {
+                fun <- match.fun(fun)
+                locs <- which(as.logical(fun(as.array(data), ...)))
+            }
+            
+            if (array)
+                return (vectorToMatrixLocs(locs, imageDims))
+            else
+                return (locs)
+        }
     },
     
     getData = function () { return (data) },
@@ -140,7 +195,7 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         
         dim <- getDimensionality()
         loc <- resolveVector(len=dim, ...)
-        if (is.null(loc) || (length(...) != dim))
+        if (is.null(loc) || (length(loc) != dim))
             report(OL$Error, "Point must be specified as a ", dim, "-vector")
             
         if (all(loc >= 1) && all(loc <= getDimensions()))
@@ -161,7 +216,7 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         if (.self$isEmpty())
             return (.self$copy())
         else
-            return (MriImage$new(imageDims=imageDims, voxelDims=voxelDims, voxelDimUnits=voxelDimUnits, source=source, origin=origin, storedXform=storedXform, reordered=reordered, tags=tags, data=NULL))
+            return (MriImage$new(imageDims=imageDims, voxelDims=voxelDims, voxelDimUnits=voxelDimUnits, source=source, origin=origin, xform=xform, reordered=reordered, tags=tags, data=NULL))
     },
     
     getNonzeroIndices = function (array = TRUE, positiveOnly = FALSE)
@@ -169,25 +224,10 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         "Find voxels whose values are not zero"
         .warnIfIndexingUnreorderedImage(.self)
         
-        if (.self$isEmpty())
-            report(OL$Error, "The image contains no data")
-        else if (.self$isSparse())
-        {
-            locs <- data$getCoordinates()
-            if (positiveOnly)
-                locs <- locs[data$getData() > 0,]
-            if (array)
-                return (locs)
-            else
-                return (matrixToVectorLocs(locs, data$getDimensions()))
-        }
+        if (positiveOnly)
+            return (.self$find(fx(x > 0), array=array))
         else
-        {
-            if (positiveOnly)
-                return (which(data > 0, arr.ind=array))
-            else
-                return (which(data != 0, arr.ind=array))
-        }
+            return (.self$find(array=array))
     },
     
     getOrigin = function () { return (origin) },
@@ -250,27 +290,25 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
     getXform = function (implicit = TRUE)
     {
         "Retrieve the stored or implicit xform matrix"
-        if (!.self$isReordered() && equivalent(dim(storedXform),c(4,4)))
-            return (storedXform)
-        else if (implicit)
+        if (is.emptyMatrix(.self$xform) && .self$isReordered() && implicit)
         {
-            xform <- diag(4)
+            implicitXform <- diag(4)
             zeroBasedOrigin <- pmax(origin-1, c(0,0,0))
             if (.self$getDimensionality() == 2)
             {
-                xform[c(1,6)] <- c(-1,1) * abs(voxelDims)
+                implicitXform[c(1,6)] <- c(-1,1) * abs(voxelDims)
                 zeroBasedOrigin[1:2] <- zeroBasedOrigin[1:2] * abs(voxelDims)
             }
             else
             {
-                xform[c(1,6,11)] <- c(-1,1,1) * abs(voxelDims[1:3])
+                implicitXform[c(1,6,11)] <- c(-1,1,1) * abs(voxelDims[1:3])
                 zeroBasedOrigin <- zeroBasedOrigin * abs(voxelDims[1:3])
             }
-            xform[1:3,4] <- c(1,-1,-1) * zeroBasedOrigin
-            return (xform)
+            implicitXform[1:3,4] <- c(1,-1,-1) * zeroBasedOrigin
+            return (implicitXform)
         }
         else
-            return (emptyMatrix())
+            return (.self$xform)
     },
     
     hasTags = function (keys) { return (sapply(keys, function(key) !is.null(tags[[key]]))) },
@@ -302,8 +340,7 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         if (isTRUE(sparse))
             result <- as(result, "SparseArray")
         
-        .self$data <- result
-        .self$setSource(NULL)
+        .self$setData(result)
     },
     
     mask = function (maskImage)
@@ -318,6 +355,19 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
     
     nVolumes = function () { return (ifelse(length(imageDims) > 3, prod(imageDims[-(1:3)]), 1L)) },
     
+    setData = function (newData)
+    {
+        "Replace the data in the image"
+        if (is.null(newData) || equivalent(dim(newData),imageDims))
+        {
+            .self$data <- newData
+            .self$setSource(NULL)
+        }
+        else
+            flag(OL$Warning, "New data does not match the image dimensions")
+        invisible(.self)
+    },
+    
     setOrigin = function (newOrigin)
     {
         "Update the origin of the image"
@@ -339,12 +389,20 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
         invisible(.self)
     },
     
+    setTags = function (...)
+    {
+        "Add or replace metadata tags"
+        newTags <- deduplicate(list(...), .self$tags)
+        .self$tags <- newTags[!sapply(newTags,is.null)]
+        invisible(.self)
+    },
+    
     setXform = function (newXform)
     {
         "Update the xform matrix associated with the image"
         if (is.matrix(newXform) && equivalent(dim(newXform),c(4,4)))
         {
-            .self$storedXform  <- newXform
+            .self$xform  <- newXform
             .self$setSource(NULL)
         }
         invisible(.self)
@@ -390,7 +448,7 @@ MriImage <- setRefClass("MriImage", contains="SerialisableObject", fields=list(i
 
 # Register deserialiser for MriImageMetadata legacy class
 registerDeserialiser("MriImageMetadata", function (fields) {
-    object <- MriImage$new(imageDims=fields$imagedims, voxelDims=fields$voxdims, voxelDimUnits=fields$voxunit, source=fields$source, origin=fields$origin, storedXform=fields$storedXform, tags=fields$tags, data=NULL)
+    object <- MriImage$new(imageDims=fields$imagedims, voxelDims=fields$voxdims, voxelDimUnits=fields$voxunit, source=fields$source, origin=fields$origin, xform=fields$storedXform, tags=fields$tags, data=NULL)
     return (object)
 })
 
@@ -398,10 +456,12 @@ setAs("MriImage", "array", function (from) as(from$getData(),"array"))
 
 setAs("array", "MriImage", function (from) asMriImage(from))
 
+setAs("character", "MriImage", function (from) readImageFile(from))
+
 .warnIfIndexingUnreorderedImage <- function (image)
 {
     # The argument is an unreordered image and contains a non-LAS xform
-    if (is(image,"MriImage") && !image$isReordered() && xformToOrientation(image$getXform()) != "LAS")
+    if (is(image,"MriImage") && !image$isReordered() && orientation(image) != "LAS")
         flag(OL$Warning, "Indexing into an image which is not reordered has no consistent meaning")
 }
 
@@ -660,14 +720,14 @@ asMriImage <- function (data, templateImage = nilObject(), imageDims = NA, voxel
     else
         report(OL$Error, "No information on image dimensions is available")
     
-    defaults <- list(voxelDims=rep(1,nDims), voxelDimUnits="unknown", origin=c(1,1,1), storedXform=emptyMatrix(), tags=list(), reordered=TRUE)
+    defaults <- list(voxelDims=rep(1,nDims), voxelDimUnits="unknown", origin=c(1,1,1), xform=emptyMatrix(), tags=list(), reordered=TRUE)
     template <- templateImage$serialise()
     params <- list(imageDims=imageDims, voxelDims=voxelDims, voxelDimUnits=voxelDimUnits, origin=origin, tags=tags, reordered=reordered)
     params <- params[!is.na(params)]
     
     composite <- deduplicate(params, template, defaults)
     
-    image <- MriImage$new(imageDims=composite$imageDims[1:nDims], voxelDims=composite$voxelDims[1:nDims], voxelDimUnits=composite$voxelDimUnits, origin=composite$origin, storedXform=composite$storedXform, reordered=composite$reordered, tags=composite$tags, data=data)
+    image <- MriImage$new(imageDims=composite$imageDims[1:nDims], voxelDims=composite$voxelDims[1:nDims], voxelDimUnits=composite$voxelDimUnits, origin=composite$origin, xform=composite$xform, reordered=composite$reordered, tags=composite$tags, data=data)
     invisible (image)
 }
 
@@ -675,13 +735,12 @@ asMriImage <- function (data, templateImage = nilObject(), imageDims = NA, voxel
 #' @export
 extractMriImage <- function (image, dim, loc)
 {
-    if (!is(image, "MriImage"))
-        report(OL$Error, "The specified image is not an MriImage object")
+    image <- as(image, "MriImage")
     
     newData <- image$getSlice(dim, loc)
     dimsToKeep <- setdiff(1:image$getDimensionality(), dim)
     
-    image <- MriImage$new(imageDims=image$getDimensions()[dimsToKeep], voxelDims=image$getVoxelDimensions()[dimsToKeep], voxelDimUnits=image$getVoxelUnits(), origin=image$getOrigin(), storedXform=image$getXform(implicit=FALSE), reordered=image$isReordered(), tags=image$getTags(), data=newData)
+    image <- MriImage$new(imageDims=image$getDimensions()[dimsToKeep], voxelDims=image$getVoxelDimensions()[dimsToKeep], voxelDimUnits=image$getVoxelUnits(), origin=image$getOrigin(), xform=image$getXform(implicit=FALSE), reordered=image$isReordered(), tags=image$getTags(), data=newData)
     return (image)
 }
 
@@ -689,8 +748,8 @@ extractMriImage <- function (image, dim, loc)
 #' @export
 trimMriImage <- function (image, clearance = 4, indices = NULL)
 {
-    if (!is(image, "MriImage"))
-        report(OL$Error, "The specified image is not an MriImage object")
+    image <- as(image, "MriImage")
+    
     if (length(clearance) == 1)
         clearance <- rep(clearance, image$getDimensionality())
     
@@ -721,97 +780,14 @@ trimMriImage <- function (image, clearance = 4, indices = NULL)
 #' @export
 reorderMriImage <- function (image)
 {
-    if (!is(image, "MriImage"))
-        report(OL$Error, "The specified image is not an MriImage object")
-    
-    # Image is already reordered
-    if (image$isReordered())
+    if (is(image,"MriImage") && image$isReordered())
         return (image)
-    
-    xformMatrix <- image$getXform(implicit=FALSE)
-    
-    # There is no xform matrix stored with the image - we can't do anything
-    if (!equivalent(dim(xformMatrix), c(4,4)))
-        return (image)
-    
-    data <- image$getData()
-    dims <- image$getDimensions()
-    voxelDims <- abs(image$getVoxelDimensions())
-    nDims <- image$getDimensionality()
-    origin <- image$getOrigin()
-    
-    # Extract the 3x3 matrix which relates to rotation
-    rotationMatrix <- extractRotationMatrixFromXform(xformMatrix)
-    absRotationMatrix <- abs(rotationMatrix)
-    tolerance <- 1e-3
-    
-    # The rotation matrix should have exactly one nonzero element per row and column
-    # If not, warn but try to figure out the closest primary orientation
-    if (!equivalent(rowSums(absRotationMatrix > tolerance), c(1,1,1)) || !equivalent(colSums(absRotationMatrix > tolerance), c(1,1,1)))
+    else
     {
-        flag(OL$Warning, "The image is stored in a rotated frame of reference")
-        tolerance <- 0.5
-        if (!equivalent(rowSums(absRotationMatrix > tolerance), c(1,1,1)) || !equivalent(colSums(absRotationMatrix > tolerance), c(1,1,1)))
-            report(OL$Error, "Cannot work out the primary orientation of the image")
+        image <- retrieveNifti(image)
+        orientation(image) <- "LAS"
+        return (as(structure(image,reordered=TRUE), "MriImage"))
     }
-    
-    # Work out the permutation required to get to LAS, and apply it
-    dimPermutation <- apply(absRotationMatrix > tolerance, 1, which)
-    if (nDims > 3)
-        dimPermutation <- c(dimPermutation, 4:nDims)
-    else if (nDims < 3)
-        dimPermutation <- dimPermutation[1:nDims]
-    if (!identical(dimPermutation, seq_len(nDims)))
-    {
-        dims <- dims[dimPermutation]
-        voxelDims <- voxelDims[dimPermutation]
-        origin <- origin[dimPermutation]
-        
-        if (!image$isEmpty())
-        {
-            if (image$isSparse())
-                data$aperm(dimPermutation)
-            else
-                data <- aperm(data, dimPermutation)
-        }
-    }
-    
-    # Figure out which dimensions need to be flipped
-    # We sum by row because the data dimensions have already been permuted
-    ordering <- sign(rowSums(rotationMatrix))
-    ordering <- ordering * c(-1, 1, 1)
-    
-    # Flip data and origin as required
-    indices <- 1:min(3,nDims)
-    if (any(ordering[indices] < 0))
-    {
-        origin[indices] <- ifelse(ordering[indices] < 0, dims[indices]-origin[indices]+1, origin[indices])
-        
-        if (!image$isEmpty())
-        {
-            if (image$isSparse())
-                data$flip(which(ordering[indices] < 0))
-            else
-            {
-                orderX <- (if (ordering[1] == 1) seq_len(dims[1]) else rev(seq_len(dims[1])))
-                orderY <- (if (ordering[2] == 1) seq_len(dims[2]) else rev(seq_len(dims[2])))
-                if (nDims > 2)
-                    orderZ <- (if (ordering[3] == 1) seq_len(dims[3]) else rev(seq_len(dims[3])))
-                dimsToKeep <- setdiff(1:nDims, 1:3)
-
-                if (nDims == 2)
-                    data <- data[orderX, orderY]
-                else if (nDims == 3)
-                    data <- data[orderX, orderY, orderZ]
-                else
-                    data <- array(apply(data, dimsToKeep, "[", orderX, orderY, orderZ), dim=dim(data))
-            }
-        }
-    }
-    
-    image <- MriImage$new(imageDims=dims, voxelDims=voxelDims, voxelDimUnits=image$getVoxelUnits(), source=image$getSource(), origin=origin, storedXform=xformMatrix, reordered=TRUE, tags=image$getTags(), data=data)
-    
-    return (image)
 }
 
 #' Merging MriImage objects
@@ -821,10 +797,19 @@ reorderMriImage <- function (image)
 #' image and returns it.
 #' 
 #' @param ... \code{MriImage} objects. They do not need to have the same
-#'   dimensionality, but they would usually not vary by more than one
-#'   dimension.
+#'   dimensionality.
+#' @param bindDim An integer specifying the dimension along which to bind the
+#'   data, or \code{NULL} (the default). The latter case resolves to one number
+#'   higher than the last dimension common to all images.
+#' @param padTags Logical value. If \code{TRUE}, \code{NA}s will be used to pad
+#'   tags which appear to be partially missing in the merged dataset. If
+#'   \code{FALSE}, incomplete tags will be dropped.
 #' @return A merged image.
 #' 
+#' @note Tags are retained as-is if they are identical in each image. Otherwise
+#'   they are concatenated if their lengths match the number of blocks in each
+#'   image, or concatenated with NAs for missing values if \code{padTags} is
+#'   \code{TRUE}.
 #' @author Jon Clayden
 #' @seealso \code{\linkS4class{MriImage}}
 #' @references Please cite the following reference when using TractoR in your
@@ -835,42 +820,70 @@ reorderMriImage <- function (image)
 #' Journal of Statistical Software 44(8):1-18.
 #' \url{http://www.jstatsoft.org/v44/i08/}.
 #' @export
-mergeMriImages <- function (...)
+mergeMriImages <- function (..., bindDim = NULL, padTags = FALSE)
 {
-    images <- list(...)
-    if (any(!sapply(images, is, "MriImage")))
-        report(OL$Error, "All arguments must be MriImage objects")
+    images <- lapply(list(...), as, "MriImage")
     if (length(images) == 1)
         return (images[[1]])
+    if (!allEqual(sapply(images, orientation)))
+        images <- lapply(images, reorderMriImage)
+    if (!allEqual(lapply(images, xform), tolerance=1e-4))
+        report(OL$Warning, "Merging images with nonequal xforms - this is probably unwise")
     
     dimensionalities <- sapply(images, function(x) x$getDimensionality())
-    dimensions <- sapply(seq_along(images), function(i) c(images[[i]]$getDimensions(), rep(NA,max(dimensionalities)-dimensionalities[i])))
+    lastDim <- max(dimensionalities, bindDim)
+    dimensions <- sapply(seq_along(images), function(i) c(images[[i]]$getDimensions(), rep(1L,lastDim-dimensionalities[i])))
     
     commonDims <- apply(dimensions, 1, allEqual)
-    if (!commonDims[1])
-        report(OL$Error, "Images must have at least their first dimension in common")
-    commonDims[length(commonDims)] <- FALSE
-    lastCommonDim <- which(!commonDims)[1] - 1L
-    blockDims <- dimensions[1:lastCommonDim,1]
+    assert(commonDims[1], "Images must have at least their first dimension in common")
+    if (is.null(bindDim))
+        bindDim <- max(which(commonDims)) + 1L
+    if (bindDim > lastDim)
+        dimensions <- rbind(dimensions, matrix(1,nrow=(bindDim-lastDim),ncol=ncol(dimensions)))
+    blockDims <- dimensions[seq_len(bindDim-1),1]
+    blockCounts <- apply(dimensions[bindDim:lastDim,,drop=FALSE], 2, prod)
     
     imageSizes <- apply(dimensions, 2, prod, na.rm=TRUE)
     data <- do.call("c", lapply(images, as.array))
     dim(data) <- c(blockDims, length(data) %/% prod(blockDims))
     
+    resolveValue <- function (value, requiredLength)
+    {
+        if (is.null(value))
+            rep(NA, requiredLength)
+        else if (length(value) == requiredLength)
+            value
+        else if (nrow(promote(value,TRUE)) == requiredLength)
+            promote(value, TRUE)
+        else if (length(value) == 1)
+            rep(value, requiredLength)
+        else
+            NULL
+    }
+    
+    tags <- lapply(seq_along(images), function(i) c(list(.blocks=blockCounts[i]), images[[i]]$getTags()))
+    tagNames <- Reduce(ifelse(padTags,union,intersect), lapply(tags, names))
     tags <- Reduce(function(x,y) {
-        sapply(intersect(names(x$getTags()),names(y$getTags())), function(n) {
-            tx <- x$getTags(n)
-            ty <- y$getTags(n)
-            if (equivalent(tx, ty))
-                tx
-            else if (length(dim(data)) == 3 && length(tx) == x$nSlices() && length(ty) == y$nSlices())
-                c(tx, ty)
-            else if (length(dim(data)) == 4 && length(tx) == x$nVolumes() && length(ty) == y$nVolumes())
-                c(tx, ty)
+        sapply(tagNames, function(n) {
+            if (n == ".blocks")
+                x[[n]] + y[[n]]
+            else if (equivalent(x[[n]], y[[n]]))
+                x[[n]]
             else
-                NULL
+            {
+                xn <- resolveValue(x[[n]], x$.blocks)
+                yn <- resolveValue(y[[n]], y$.blocks)
+                if (is.matrix(xn) || is.matrix(yn))
+                    rbind(xn, yn)
+                else if (is.null(xn) || is.null(yn))
+                    NULL
+                else
+                    c(xn, yn)
+            }
         }, simplify=FALSE, USE.NAMES=TRUE)
-    }, images)
+    }, tags)
+    
+    tags$.blocks <- NULL
     
     return (asMriImage(data, images[[which.max(imageSizes)]], tags=tags[!sapply(tags,is.null)]))
 }
