@@ -12,51 +12,54 @@ readMgh <- function (fileNames)
     if (version != 1)
         report(OL$Error, "Only version 1 MGH/MGZ files are supported")
     
+    # Create a default header
+    header <- niftiHeader()
+    
     dims <- readBin(connection, "integer", n=4, size=4, endian="big")
     typeCode <- readBin(connection, "integer", n=1, size=4, endian="big")
     readBin(connection, "raw", n=4)
     orientationStored <- as.logical(readBin(connection, "integer", n=1, size=2, endian="big"))
     voxelDims <- readBin(connection, "double", n=3, size=4, endian="big")
     
+    scaleMatrix <- diag(c(abs(voxelDims[1:3]), 1))
     if (orientationStored)
     {
-        xformMatrix <- matrix(readBin(connection, "double", n=12, size=4, endian="big"), nrow=3)
-        xformMatrix <- rbind(xformMatrix, c(0,0,0,1))
+        # The stored xform is not scaled by the voxel dimensions
+        xform <- matrix(readBin(connection, "double", n=12, size=4, endian="big"), nrow=3)
+        xform <- rbind(xform, c(0,0,0,1)) %*% scaleMatrix
     }
     else
-        xformMatrix <- matrix(c(-1,0,0,0,0,0,-1,0,0,1,0,0,0,0,0,1), nrow=4)
+    {
+        # Default orientation is LIA
+        xform <- scaleMatrix
+        orientation(xform) <- "LIA"
+    }
+    
+    # Whether the information is stored or not, an origin adjustment is needed,
+    # because MGH format uses the image centre as a reference point
+    xform[1:3,4] <- xform[1:3,4] - (xform[1:3,1:3] %*% (dims[1:3] / 2))
     
     close(connection)
     
-    absRotationMatrix <- abs(xformMatrix[1:3,1:3])
-    tolerance <- 1e-3 * max(abs(voxelDims))
-    
-    # The rotation matrix should have exactly one nonzero element per row and column
-    if (!equivalent(rowSums(absRotationMatrix > tolerance), c(1,1,1)) || !equivalent(colSums(absRotationMatrix > tolerance), c(1,1,1)))
-        report(OL$Error, "The image is stored in a rotated frame of reference")
-    
-    # Locate the nonzero elements of the rotation matrix
-    indices <- cbind(1:3, apply(absRotationMatrix > tolerance, 1, which))
-    xformMatrix[indices] <- xformMatrix[indices] * abs(voxelDims)
-    xformMatrix[1:3,4] <- xformMatrix[1:3,4] - c(dims[1]/2,dims[2]/2,dims[3]/2)[indices[,2]] * xformMatrix[indices]
+    nDims <- max(which(dims > 1))
+    if (nDims < 2)
+        report(OL$Error, "MGH image appears to have less than two non-unitary dimensions")
+    header$dim[seq_len(nDims+1)] <- c(nDims, dims[1:nDims])
+    qform(header) <- structure(xform, code=2L)
+    header$pixdim[seq_len(nDims)+1] <- voxelDims[1:nDims]
     
     typeIndex <- which(.Mgh$datatypes$codes == typeCode)
     if (length(typeIndex) != 1)
         report(OL$Error, "The MGH data type code is not valid")
     datatype <- list(code=typeCode, type=.Mgh$datatypes$rTypes[typeIndex], size=.Mgh$datatypes$sizes[typeIndex], isSigned=.Mgh$datatypes$isSigned[typeIndex])
     
-    dimsToKeep <- 1:max(which(dims > 1))
-    imageMetadata <- list(imageDims=dims[dimsToKeep], voxelDims=voxelDims[dimsToKeep], voxelUnit=NULL, source=fileNames$fileStem, tags=list())
-    
-    storageMetadata <- list(dataOffset=284, dataScalingSlope=1, dataScalingIntercept=0, xformMatrix=xformMatrix, datatype=datatype, endian="big")
-    
-    invisible (list(imageMetadata=imageMetadata, storageMetadata=storageMetadata))
+    storage <- list(offset=284, slope=1, intercept=0, datatype=datatype, endian="big")
+    invisible (list(image=NULL, header=header, storage=storage))
 }
 
 writeMgh <- function (image, fileNames, gzipped = FALSE)
 {
-    if (!is(image, "MriImage"))
-        report(OL$Error, "The specified image is not an MriImage object")
+    image <- as(image, "MriImage")
     
     fileFun <- (if (gzipped) gzfile else file)
     
@@ -74,8 +77,7 @@ writeMgh <- function (image, fileNames, gzipped = FALSE)
     
     fullVoxelDims <- c(image$getVoxelDimensions(), rep(0,3-ndims))[1:3]
     
-    if (!image$isReordered())
-        report(OL$Error, "Unreordered images cannot currently be written to MGH/MGZ format")
+    assert(image$isReordered(), "Unreordered images cannot currently be written to MGH/MGZ format")
     
     # We can assume the image is reordered here
     origin <- image$getXform()[1:3,4] + fullDims[1:3]/2 * fullVoxelDims * c(-1,1,1)
